<?php

namespace Drupal\user_provisioning\EntityHandler;

use Drupal;
use Drupal\Core\Entity\EntityInterface;
use Drupal\user\Entity\User;
use Drupal\user_provisioning\APIHandler\moUserProvisioningAPIHandler;
use Drupal\user_provisioning\Helpers\moUserProvisioningAudits;
use Drupal\user_provisioning\Helpers\moUserProvisioningLogger;
use Drupal\user_provisioning\SCIMParser\moUserProvisioningUserParser;
use Exception;
use Psr\Log\LoggerInterface;

class moUserProvisioningUserHandler implements moUserProvisioningEntityHandlerInterface
{

    /**
     * @var EntityInterface
     */
    private EntityInterface $entity;

    private moUserProvisioningUserParser $resource_parser;

    private moUserProvisioningAPIHandler $apiHandler;

    private moUserProvisioningAudits $audits;

    private LoggerInterface $logger;

    private moUserProvisioningLogger $mo_logger;


    /**
     * @param EntityInterface $entity User or Role entity
     */
    public function __construct(EntityInterface $entity)
    {
        $this->entity = $entity;
        $this->resource_parser =  new moUserProvisioningUserParser();
        $this->apiHandler = new moUserProvisioningAPIHandler();
        $this->audits = new moUserProvisioningAudits();
        $this->logger = Drupal::logger('user_provisioning');
        $this->mo_logger = new moUserProvisioningLogger();
    }

    /**
     * @inheritDoc
     */
    public function getResource()
    {
        // TODO: Implement getResource() method.
    }

    /**
     * @inheritDoc
     * @throws Exception
     */
    public function createOrUpdateResource()
    {
        list($status_code, $content) = $this->searchResource();
        if (is_null($content)){
            $this->mo_logger->addLog('Received searchResponse is null.', __LINE__, __FUNCTION__, __FILE__);
            return null;
        }

        // TODO: Implement createOrUpdateResource() method. Send the api request for creating user, if not succeeded then add the operation in the Queue factory to be processed later.

        if ($status_code == 200) {
            $response_body = json_decode($content, true);
            if (isset($response_body['totalResults'])) {
                if ($response_body['totalResults'] == 0) {
                    //make api call to create the resource
                    try {
                        $create_response = $this->apiHandler->post($this->resource_parser->post($this->getUser()));
                        $create_content = $create_response->getBody()->getContents();

                        $this->mo_logger->addFormattedLog(json_decode($create_content, true), __LINE__, __FUNCTION__, __FILE__, 'The received user create response is:');
                        $this->addUserAudit('CREATE', $create_response->getStatusCode());

                        return $create_content;
                    } catch (Exception $exception) {
                        $this->logger->error( __FUNCTION__ . ': ' . t($exception->getMessage()));
                        $this->addUserAudit('CREATE');
                        return null;
                    }
                    // TODO process the POST response, store the received user id to db for future reference when you have to make a put or delete operation
                } else {
                    $this->addUserAudit('CREATE');
                    throw new Exception('Conflict. User already exists at the configured application.');
                }
            } else{
                $this->addUserAudit('CREATE');
                throw new Exception('The SERVER has returned an unexpected result. Refer to Drupal logs for more information.');
            }
        }
        elseif ($status_code == 409){
            //FIXME could not get a response that had status as conflict, need to check again
            //TODO conflict occurred, in this case send a PUT request to update the user field
            $this->addUserAudit('CREATE');
            throw new Exception('Conflict. User already exists at the configured application.');
        }
    }

    /**
     * @inheritDoc
     */
    public function searchResource()
    {
        $query = $this->resource_parser->search($this->getUser()->getEmail());
        try {
            $result = $this->apiHandler->get($query);
            $status_code = $result->getStatusCode();
            $content = $result->getBody()->getContents();
            $this->mo_logger->addFormattedLog(json_decode($content, true), __LINE__, __FUNCTION__, __FILE__, 'The received user search response is:');
            $this->addUserAudit('READ',  $status_code);
            return [$status_code, $content];
        } catch (Exception $exception) {
            $this->logger->error( __FUNCTION__ . ': ' . t($exception->getMessage()));
            $this->addUserAudit('READ');
            return [$exception->getCode(), $exception->getMessage()];
        }
    }

    private function getUser(): User
    {
        return User::load($this->entity->id());
    }

    /**
     * @inheritDoc
     */
    public function deleteResource()
    {
        // TODO: Implement deleteResource() method. Send the api request for deleting user, if not succeeded then add the operation in the Queue factory to be processed later.
    }

    /**
     * @param string $operation Operation name
     * @param int $status_code Status code of the executed operation. Default value handles the case "FAILED".
     * @return void
     */
    private function addUserAudit(string $operation , int $status_code = -1){
        $this->audits->addAudit([$this->entity->id(),
            $this->entity->getAccountName(),
            time(),
            $operation,
            $status_code == -1 ? 'FAILED' : 'SUCCESS']);
    }
}