<?php

namespace Drupal\user_provisioning\Form;

use Drupal;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\user\Entity\User;
use Drupal\user_provisioning\EntityHandler\moUserProvisioningUserHandler;
use Drupal\user_provisioning\Helpers\moProviderSpecificProvisioning;
use Drupal\user_provisioning\Helpers\moUserProvisioningLogger;
use Drupal\user_provisioning\moUserProvisioningConstants;
use Drupal\user_provisioning\moUserProvisioningOperationsHandler;
use Drupal\user_provisioning\moUserProvisioningUtilities;
use Exception;
use Psr\Log\LoggerInterface;

class MoUserProvisioning extends FormBase
{
    protected $messenger;
    private $base_url;
    private ImmutableConfig $config;
    private Config $config_factory;
    private LoggerInterface $logger;
    private moUserProvisioningLogger $mo_logger;

    public function __construct()
    {
        global $base_url;
        $this->base_url = $base_url;
        $this->config = Drupal::config('user_provisioning.settings');
        $this->config_factory = Drupal::configFactory()->getEditable('user_provisioning.settings');
        $this->logger = Drupal::logger('user_provisioning');
        $this->messenger = Drupal::messenger();
        $this->mo_logger = new moUserProvisioningLogger();
    }

    /**
     * @inheritDoc
     */
    public function getFormId(): string
    {
        return "mo_user_provisioning";
    }

    /**
     * @inheritDoc
     */
    public function buildForm(array $form, FormStateInterface $form_state): array
    {
        $form['mo_user_provisioning_add_css'] = array(
            '#attached' => array(
                'library' => array(
                    'user_provisioning/user_provisioning.admin',
                    'user_provisioning/user_provisioning.mo_search_field',
                )
            ),
        );

        //below section is for creating the horizontal tab.
        $mo_table_content = [[
            Markup::create('<a href="' . $this->base_url . moUserProvisioningConstants::USER_PROVISIONING . '"> <strong>SCIM CLIENT<strong> </a>'),
            Markup::create('<a href="' . $this->base_url . moUserProvisioningConstants::USER_PROVISIONING . '?tab_name=' . moUserProvisioningConstants::SCIM_SERVER_TAB_NAME . '"> <strong>SCIM SERVER<strong> </a>'),
            Markup::create('<a href="' . $this->base_url . moUserProvisioningConstants::USER_PROVISIONING . '?tab_name=' . moUserProvisioningConstants::PROVIDER_SPECIFIC_PROVISIONING_TAB_NAME . '"> <strong>PROVIDER SPECIFIC PROVISIONING<strong> </a>')
        ]];

        $form['mo_user_provisioning_navbar'] = array(
            '#markup' => t('<div class="mo_user_provisioning_table_layout">'),
        );

        $form['mo_user_provisioning_table'] = array(
            '#type' => 'table',
            '#responsive' => TRUE,
            '#rows' => $mo_table_content,
            '#attributes' => ['style' => 'border-collapse: separate;', 'id' => 'user_provisioning_headers', 'class' => ['border_fill']],
        );

        $form['mo_user_provisioning_navbar_1'] = array(
            '#markup' => t('<div class="mo_user_provisioning_container_user_provisioning_tab">'),
        );

        //horizontal tab section ends here

        $tab_name = $this->getTabName();

        if ($tab_name == moUserProvisioningConstants::SCIM_SERVER_TAB_NAME) {
            $this->getSCIMServerForm($form);
        } elseif ($tab_name == moUserProvisioningConstants::PROVIDER_SPECIFIC_PROVISIONING_TAB_NAME) {
            $this->getProviderSpecificForm($form);
        } else {
            $this->getSCIMClientForm($form);
        }

        return $form;
    }

    /**
     * Fetches and return the name of the tab from the get parameter
     */
    private function getTabName()
    {
        return Drupal::request()->get('tab_name');
    }

    /**
     * Generates and returns the form for the SCIM SERVER configuration.
     * @param array $form
     * @return void
     */
    private function getSCIMServerForm(array &$form)
    {
        $url = $this->base_url . moUserProvisioningConstants::UPGRADE_PLANS;
        $mo_premium_tag = '<a href= "' . $url . '" >[PREMIUM]</a>';

        $mo_table_content = array(
            array('SCIM Base URL', $this->t('This feature is available in the <a href="'. $this->base_url . moUserProvisioningConstants::UPGRADE_PLANS .'">Premium</a> version of the module.')),
            array('SCIM Bearer Token', $this->t('This feature is available in the <a href="'. $this->base_url . moUserProvisioningConstants::UPGRADE_PLANS .'">Premium</a> version of the module.')),
        );

        $form['mo_user_provisioning_scim_server_configuration'] = array(
            '#type' => 'fieldset',
            '#title' => t('CONFIGURE DRUPAL AS A SCIM SERVER: ') . $mo_premium_tag.'<hr>',
            '#attributes' => array('style' => 'padding:2% 2% 4%; margin-bottom:2%'),
        );

        $form['mo_user_provisioning_scim_server_configuration']['mo_user_provisioning_scim_server_meatadata'] = array(
            '#type' => 'table',
            '#header' => array('ATTRIBUTE', 'VALUE'),
            '#rows' => $mo_table_content,
            '#empty' => t('Something is not right. Please run the update script or contact us at <a href="mailto:drupalsupport@xecurify.com">drupalsupport@xecurify.com</a>'),
            '#responsive' => TRUE,
            '#sticky' => TRUE,
            '#size' => 2,
            '#prefix' => '<br>',
        );

        $form['mo_user_provisioning_scim_server_configuration']['mo_user_provisioning_scim_server_generate_new_token'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Generate new Token'),
            '#disabled' => true,
            '#prefix' => '<br/>',
            '#suffix' => '<br/>',
        );

        $form['mo_user_provisioning_scim_server_configuration']['mo_user_provisioning_scim_server_description'] = array(
            '#markup' => t('</br></br>
                <div class="mo_user_provisioning_tab_sub_heading">This tab is capable of doing following SCIM Operations <strong>' . $mo_premium_tag . '</strong></div><hr>
                <br><div class="mo_user_provisioning_background_note">
                    <strong>Create:</strong>
                    <ol>It will create users using usernames and other attributes of IDP.<br/>
                    <strong>Note: </strong>If Username field is blank, it will copy email as a username, as Drupal does not accept blank Username.</ol>
                    <strong>Update:</strong>
                    <ol>It will update the user fields, all attributes/fields except email and username.</ol>
                    <strong>Delete:</strong>
                    <ol>Once a user is deleted from IDP, it would delete a user from Drupal User list as well.</ol>
                    <strong>Deactivate:</strong>
                    <ol>Once a user is deleted from IDP, it would deactivate a user from the Drupal user list.</ol>
                </div>
                </div>')
        );
    }

    /**
     * Generates and returns the form for the Provider Specific Provisioning configuration.
     * @param array $form
     * @return void
     */
    private function getProviderSpecificForm(array &$form)
    {
        $form['mo_user_provisioning_text_search'] = [
            '#type' => 'textfield',
            '#prefix' => '<div class="mo_user_provisioning_search_provider"><br/>',
            '#title' => $this->t('Search Provider/Application'),
            '#placeholder' => $this->t('Search your Provider'),
            '#attributes' => [
                'id' => 'mo_text_search',
                'onkeyup' => 'searchApp()',
            ],
            '#suffix' => '</div><br>',
        ];

        $provider_specific_provisioning = new moProviderSpecificProvisioning();
        $api_providers = $provider_specific_provisioning->providerList();

        $form['mo_user_provisioning_application_list'] = array(
            '#prefix' => '<ul id="mo_search_ul" class="mo_user_provisioning_wrap mo_user_provisioning_flex_container"><li class="mo_user_provisioning_flex_item disabled">',
            '#markup' => implode('</li><li class="mo_user_provisioning_flex_item disabled">', $api_providers),
            '#suffix' => '</li></ul> ',
        );

    }

    /**
     * Generates and returns the form for the SCIM CLIENT configuration.
     * @param array $form
     * @return void
     */
    private function getSCIMClientForm(array &$form)
    {

        $url = $this->base_url . moUserProvisioningConstants::UPGRADE_PLANS;
        $mo_premium_tag = '<a href= "' . $url . '" >[PREMIUM]</a>';

        $form['mo_user_provisioning_config_fieldset'] = array(
            '#type' => 'fieldset',
            '#title' => t('CONFIGURE DRUPAL AS A SCIM CLIENT: &nbsp;<a href="'. $this->base_url . moUserProvisioningConstants::OVERVIEW .'">[Know more]</a><hr>'),
            '#attributes' => array('style' => 'padding:2% 2% 4%; margin-bottom:1%'),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_scim_client_enable_api_integration'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('Enable SCIM CLIENT API integration'),
            '#default_value' => $this->config->get('mo_user_provisioning_scim_client_enable_api_integration'),
            '#description' => '<strong>Note:-</strong> ' . $this->t('Enable this checkbox to update below configuration and activate the provisioning.'),
            '#prefix' => '<br/>',
            '#suffix' => '<br/>',
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_scim_server_base_url'] = array(
            '#type' => 'url',
            '#title' => $this->t('SCIM 2.0 Base Url') . '&emsp;&emsp;&ensp;',
            '#default_value' => $this->config->get('mo_user_provisioning_scim_server_base_url'),
            '#wrapper_attributes' => array(
                'class' => 'mo_user_provisioning_inline_wrapper'
            ),
            '#attributes' => array('placeholder' => $this->t('Base URL of the SCIM Server Applicaion')),
            '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_scim_server_bearer_token'] = array(
            '#type' => 'textfield',
            '#title' => $this->t('SCIM Bearer Token') . '&emsp;&emsp;',
            '#maxlength' => 1024,
            '#default_value' => $this->config->get('mo_user_provisioning_scim_server_bearer_token'),
            '#wrapper_attributes' => array(
                'class' => 'mo_user_provisioning_inline_wrapper'
            ),
            '#attributes' => array('placeholder' => $this->t('Bearer Token')),
            '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
            '#suffix' => '<br/><b>Enable Provisioning Features: </b>' . $mo_premium_tag,
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_create_user'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('Create User '),
            '#disabled' => true,
            '#default_value' => $this->config->get('mo_user_provisioning_create_user'),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_update_user'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('Update User '),
            '#disabled' => true,
            '#default_value' => $this->config->get('mo_user_provisioning_update_user'),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_delete_user'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('Delete User '),
            '#disabled' => true,
            '#default_value' => $this->config->get('mo_user_provisioning_delete_user'),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_deactivate_user'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('Deactivate User '),
            '#disabled' => true,
            '#default_value' => $this->config->get('mo_user_provisioning_deactivate_user'),
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_role_provisioning'] = array(
            '#type' => 'checkbox',
            '#title' => $this->t('<strong>Enable Role Provisioning </strong>') . $mo_premium_tag,
            '#disabled' => true,
            '#default_value' => $this->config->get('mo_user_provisioning_role_provisioning'),
            '#prefix' => '<br/>',
        );

        $form['mo_user_provisioning_config_fieldset']['mo_user_provisioning_configuration_test'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => $this->t('Save and Test Credentials'),
            '#submit' => array('::testAPICredentials'),
            '#prefix' => '<br/>',
            '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
        );

        $form['mo_user_provisioning_scim_client_mapping'] = array(
            '#type' => 'details',
            '#title' => $this->t('Mapping'),
            '#open' => true,
            '#prefix' => '<br/>',
        );

        $form['mo_user_provisioning_scim_client_mapping']['mo_user_provisioning_scim_client_attribute_mapping_header'] = array(
            '#markup' => '<br/><strong class="mo_user_provisioning_scim_header">Attribute Mapping  </strong>' . $mo_premium_tag . '<hr><br/>
                            <div class="mo_user_provisioning_background_note"><strong>Note: </strong>Under below section, you can add the configuration to send the attributes from Drupal site to your
                            configured SCIM SERVER application. </div><br/>
                            <Strong class="mo_user_provisioning_scim_header">Add Attributes </Strong><a onclick="addButtonClick()" class="button">+</a>' . $mo_premium_tag . '<br/><br/>',
        );

        $fields = moUserProvisioningUtilities::customUserFields();
        $fields['name'] = 'Username'; //this is updated to avoid the confusion between name and username
        $form['mo_user_provisioning_scim_client_mapping']['mo_user_provisioning_scim_client_username_attribute'] = array(
            '#type' => 'select',
            '#title' => t('Username Attribute'),
            '#options' => $fields,
            '#disabled' => true,
            '#default_value' => 'name',
            '#prefix' => '<div class="mo_user_provisioning_inline">',
        );

        $form['mo_user_provisioning_scim_client_mapping']['mo_user_provisioning_scim_client_username_value'] = array(
            '#type' => 'textfield',
            '#title' => t('Value'),
            '#disabled' => true,
            '#default_value' => 'username',
            '#prefix' => '<div class="mo_user_provisioning_left_margin">',
            '#suffix' => '</div></div>',
        );

        $form['mo_user_provisioning_scim_client_mapping']['mo_user_provisioning_scim_client_email_attribute'] = array(
            '#type' => 'select',
            '#title' => t('Email Attribute'),
            '#options' => $fields,
            '#disabled' => true,
            '#default_value' => 'mail',
            '#prefix' => '<div class="mo_user_provisioning_inline">',
        );

        $form['mo_user_provisioning_scim_client_mapping']['mo_user_provisioning_scim_client_email_value'] = array(
            '#type' => 'textfield',
            '#title' => t('Value'),
            '#disabled' => true,
            '#default_value' => 'email',
            '#prefix' => '<div class="mo_user_provisioning_left_margin">',
            '#suffix' => '</div></div></div>',
        );

        self::provisionOnDemandForm($form);
    }

    public function provisionOnDemandForm(array &$form){
        $url = $this->base_url . moUserProvisioningConstants::UPGRADE_PLANS;
        $mo_premium_tag = '<a href= "' . $url . '" >[PREMIUM]</a>';

        $form['mo_user_provisioning_provision_container'] = array(
            '#markup' => '<div class="mo_user_provisioning_right_table_layout mo_user_provisioning_right_container">',
        );

        $form['mo_user_provisioning_provision_on_demand'] = array(
            '#type' => 'fieldset',
            '#title' => $this->t('CHOOSE WHEN TO TRIGGER USER SYNC & PROVISIONING'),
        );

      $form['mo_user_provisioning_provision_on_demand']['mo_user_provisioning_on_demand'] = array(
        '#type' => 'checkbox',
        '#title' => $this->t('On-demand/ Manual Provisioning '),
        '#description' =>  $this->t('This will allow you to manually sync any Drupal user in your SCIM Server application. <br>You can select which of the CRUD operations you wish to perform from under the Enable Provisioning Features section'),
        '#default_value' => TRUE,
      );

        $form['mo_user_provisioning_provision_on_demand']['mo_user_provisioning_add_user_or_resource_field'] = array(
            '#type' => 'textfield',
            '#title' => $this->t('Enter the Drupal Username'),
            '#autocomplete_route_name' => 'user_provisioning.autocomplete',
            '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
        );

        $form['mo_user_provisioning_provision_on_demand']['mo_user_provisioning_provision_on_demand_submit'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#name' => 'UserProvision',
            '#value' => $this->t('Provision'),
            '#submit' => array('::provisionOnDemand'),
            '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
        );
        $form['mo_user_provisioning_provision_on_demand']['mo_user_provisioning_role_provision_on_demand'] = array(
          '#type' => 'checkbox',
          '#title' => $this->t('Enable provision on demand for roles ') . $mo_premium_tag,
          '#description' => '<strong>Note:- </strong>' . $this->t('Enable this checkbox to provision/sync your Drupal users based on their roles.'),
          '#disabled' => true,
          '#default_value' => $this->config->get('mo_user_provisioning_role_provision_on_demand'),
        );
      $form['mo_user_provisioning_provision_on_demand']['event_based_provisioning'] = array(
        '#type' => 'checkbox',
        '#title' => $this->t('Enable Event Based provisioning ') . $mo_premium_tag,
        '#description' => '<strong>Note:- </strong>' . $this->t('Enable this checkbox to perform provisioning when CRUD and related events are performed in Drupal'),
        '#disabled' => true,
      );
      $form['mo_user_provisioning_provision_on_demand']['scheduler_based_provisioning'] = array(
        '#type' => 'checkbox',
        '#title' => $this->t('Enable Scheduler Based provisioning ') . $mo_premium_tag,
        '#description' => '<strong>Note:- </strong>' . $this->t('Enable this checkbox to perform provisioning periodically i.e. sync and provisioning operations will keep on triggering themselves after regular intervals of time'),
        '#disabled' => true,
      );

      $form['mo_user_provisioning_provision_on_demand']['initiate_provisioning_now'] = array(
        '#type' => 'submit',
        '#button_type' => 'primary',
        '#value' => $this->t('Save Settings'),
        '#disabled' => true,
        '#states' => array('disabled' => array(':input[name = "mo_user_provisioning_scim_client_enable_api_integration"]' => array('checked' => FALSE),),),
      );
    }

    public function validateForm(array &$form, FormStateInterface $form_state)
    {
      $triggering_element = $form_state->getTriggeringElement();
     if($triggering_element['#name'] == 'UserProvision'){
       $entity_name = trim($form_state->getValues()['mo_user_provisioning_add_user_or_resource_field']);
       if (empty($entity_name)){
         $form_state->setErrorByName('mo_user_provisioning_add_user_or_resource_field', $this->t('Enter a valid Drupal Username.'));
       }
     }
    }

    /**
     * @inheritDoc
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $tab_name = $this->getTabName();

        if ($tab_name == moUserProvisioningConstants::SCIM_SERVER_TAB_NAME) {

        } elseif ($tab_name == moUserProvisioningConstants::PROVIDER_SPECIFIC_PROVISIONING_TAB_NAME) {

        } else {
            $this->submitSCIMClintForm($form, $form_state);
        }
    }

    /**
     * Generates and returns the form for SCIM CLIENT configuration.
     * @param array $form
     * @param FormStateInterface $form_state
     * @return void
     */
    private function submitSCIMClintForm(array &$form, FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        $enable_scim_client_api_integration = $form_values['mo_user_provisioning_scim_client_enable_api_integration'];
        $this->config_factory->set('mo_user_provisioning_scim_client_enable_api_integration', $enable_scim_client_api_integration)->save();

        $scim_server_base_url = trim($form_values['mo_user_provisioning_scim_server_base_url']);
        $scim_server_bearer_token = trim($form_values['mo_user_provisioning_scim_server_bearer_token']);
        $create_user = $form_values['mo_user_provisioning_create_user'];

        $show_success_message = false;
        if ($scim_server_base_url != $this->config->get('mo_user_provisioning_scim_server_base_url')
            || $scim_server_bearer_token != $this->config->get('mo_user_provisioning_scim_server_bearer_token')
            || $create_user != $this->config->get('mo_user_provisioning_create_user') ){
            $show_success_message = true;
        }

        if ($enable_scim_client_api_integration == true) {
            $this->config_factory
                ->set('mo_user_provisioning_scim_server_base_url', $scim_server_base_url)
                ->set('mo_user_provisioning_scim_server_bearer_token', $scim_server_bearer_token)
                ->set('mo_user_provisioning_create_user', $create_user)
                ->save();
        }
        if ($show_success_message){
            $this->messenger->addMessage($this->t('SCIM CLIENT configuration saved successfully.'));
        }
    }

    /**
     * Generates and returns the form for SCIM SERVER configuration.
     * @param array $form
     * @param FormStateInterface $form_state
     * @return void
     */
    private function submitSCIMServerForm(array &$form, FormStateInterface $form_state)
    {
    }

    /**
     * Generates and returns the form for Provider Specific Provisioning configuration.
     * @param array $form
     * @param FormStateInterface $form_state
     * @return void
     */
    private function submitProviderSpecificForm(array &$form, FormStateInterface $form_state)
    {
    }

    public function testAPICredentials(array &$form, FormStateInterface $form_state)
    {
        $this->submitSCIMClintForm($form, $form_state);
        $user_provisioner = new moUserProvisioningUserHandler(User::load(Drupal::currentUser()->id()));
        list($status_code, $content) = $user_provisioner->searchResource();

        if (!is_null($content)){
            $body = json_decode($content, true);
            if ($status_code == 409 || !is_null($body)){
                $this->logger->info('<b> '. __FUNCTION__ .':</b> ' . '<pre><code>' . print_r($content, TRUE) . '</code></pre>');
                $this->messenger->addMessage('Authorization successful. You can try Provision On-demand at the right section of this page.');
            }
            else{
                $this->logger->error('<b> '. __FUNCTION__ .':</b> ' . '<pre><code>' . print_r($content, TRUE) . '</code></pre>');
                $this->messenger->addError('Invalid Authorization.');
            }
        }
        else{
            $this->messenger->addError($this->t('Either base url or bearer token is invalid. Check'.' <a href="'. $this->base_url . moUserProvisioningConstants::DRUPAL_LOGS_PATH .'" target="_blank">Drupal logs</a> '.' for more details.'));
        }
    }

    /**
     * Provision the User/role on demand.
     * @param array $form
     * @param FormStateInterface $form_state
     * @return void
     */
    public function provisionOnDemand(array &$form, FormStateInterface $form_state)
    {
        //@TODO Provision user or role on-demand

        $entity_name = trim($form_state->getValues()['mo_user_provisioning_add_user_or_resource_field']);

        $this->mo_logger->addLog('Provisioning on demand. Entered entity name is :' . $entity_name, __LINE__, __FUNCTION__, __FILE__);

        $entity = $this->getEntityToProvision($entity_name);
        if (is_null($entity)){
            $this->messenger->addError( '"'. $entity_name . '"' .$this->t(' can not be recognized as a valid User or Role. Enter a valid entity( User or Role) to provision.'));
        }

        $mo_entity_handler = new moUserProvisioningOperationsHandler($entity);
        try {
            $result = $mo_entity_handler->insert();
            if (is_null($result)){
                $this->messenger->addError($this->t('An error occurred while provisioning the user, please refer to <a href="'. $this->base_url . moUserProvisioningConstants::DRUPAL_LOGS_PATH .'">drupal logs</a> for more information.'));
                return;
            }
            $this->messenger->addMessage($this->t($entity->label() . ' successfully created at the configured application.'));
        } catch (Exception $exception) {
            $this->messenger->addError($exception->getMessage());
            // TODO PUT operation will be called following this event to update the user fields
        }
    }

    /**
     * Checks and return the User object or Role object. Returns null if not found. (Checks for User and then Role)
     * @param string $entity_name User or Role name
     * @return EntityBase|EntityInterface|User|null
     */
    private function getEntityToProvision(string $entity_name)
    {
        $user = user_load_by_name($entity_name);
        if ($user != false) {
            return User::load($user->id());
        }
        //TODO: Below commented code is for premium version
//        $roles = Role::loadMultiple();
//        $role_names = array_keys($roles);
//        if (in_array($entity_name, $role_names)) {
//            return Role::load($roles[$entity_name]->id());
//        }
        return null;
    }
}
